import * as dntShim from "../../../_dnt.shims.js";
import { parseShareAddress } from "../../core-validators/addresses.js";
import { Crypto } from "../../crypto/crypto.js";
import { extractTemplateVariablesFromPath, globToQueryAndRegex, parseTemplate, } from "../../query/query-helpers.js";
import { notErr } from "../../util/errors.js";
const HOSTED_SHARE_TEMPLATE = `/server-settings/1.*/shares/{shareHash}/{hostType}`;
/** A server extension which reads settings from a specified share, e.g. which shares to host on the server. Settings are modified by other peers syncing with this server. */
export class ExtensionServerSettings {
    constructor(opts) {
        Object.defineProperty(this, "configReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "onCreateReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.configReplica = opts.onCreateReplica(opts.settingsShare);
        this.onCreateReplica = opts.onCreateReplica;
    }
    register(peer) {
        peer.addReplica(this.configReplica);
        const onCreateReplica = this.onCreateReplica;
        const { glob } = parseTemplate(HOSTED_SHARE_TEMPLATE);
        const { query, regex } = globToQueryAndRegex(glob);
        const configShareAddress = this.configReplica.share;
        this.configReplica.getQueryStream(query, "everything").pipeTo(new dntShim.WritableStream({
            async write(event) {
                if (event.kind === "existing" || event.kind === "success") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        const pathVariables = extractTemplateVariablesFromPath(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareHash = pathVariables["shareHash"];
                        const configHash = await Crypto.sha256base32(configShareAddress);
                        if (shareHash === configHash) {
                            return;
                        }
                        if (event.doc.text.length > 0 &&
                            notErr(parseShareAddress(event.doc.text)) &&
                            !peer.hasShare(event.doc.text)) {
                            // Add
                            const replica = onCreateReplica(event.doc.text);
                            peer.addReplica(replica);
                            console.log("Server settings:", `now hosting ${replica.share}`);
                        }
                        else if (event.doc.text.length === 0) {
                            let replicaToClose;
                            for (const replica of peer.replicas()) {
                                const rShareHash = await Crypto.sha256base32(replica.share);
                                if (rShareHash === shareHash) {
                                    replicaToClose = replica;
                                }
                            }
                            if (replicaToClose) {
                                await replicaToClose.close(true);
                                peer.removeReplica(replicaToClose);
                                console.log("Server settings:", `stopped hosting ${replicaToClose.share}`);
                            }
                        }
                    }
                }
                if (event.kind === "expire") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        // Remove
                        const pathVariables = extractTemplateVariablesFromPath(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareAddress = pathVariables["shareAddress"];
                        const replicaToClose = peer.getReplica(shareAddress);
                        if (replicaToClose) {
                            await replicaToClose.close(true);
                            peer.removeReplica(replicaToClose);
                            console.log("Server settings:", `stopped hosting ${replicaToClose.share}`);
                        }
                    }
                }
            },
        }));
        return Promise.resolve();
    }
    handler(_req) {
        return Promise.resolve(null);
    }
}
